@@ -25,7 +25,8 @@ from calculate.lib.datavars import (Variable, VariableError,
                                     TableVariable, FieldValue,
                                     HumanReadable, CriticalError,
                                     SimpleDataVars, DataVarsError)
-from calculate.lib.utils.binhosts import Binhosts, PackagesIndex, HOURS
+from calculate.lib.utils.binhosts import (Binhosts, PackagesIndex, HOURS,
+    BinhostSignError)
 from calculate.lib.utils.files import readFile, listDirectory, process, pathJoin
 
 from calculate.lib.configparser import ConfigParser
@@ -41,6 +42,7 @@ from calculate.lib.variables import env
 from calculate.update.update_info import UpdateInfo
 from itertools import chain
 from urlparse import urlparse
+import io
 import time
 
 _ = lambda x: x
@@ -277,6 +279,25 @@ class VariableClUpdateBranchRep(ReadonlyVariable):
     def get(self):
         return self.Get('cl_update_rep_name')
 
+class VariableClUpdateBinhostsInfo(ReadonlyVariable):
+    """
+    Объект для получения информации о серверах обновлений
+    """
+    type = Variable.Types.Object
+    
+    def get(self):
+        last_ts = self.Get('cl_update_last_timestamp')
+        if self.GetBool('cl_update_binhost_stable_opt_set'):
+            binhost_list = self.Get('cl_update_binhost_list')
+        else:
+            binhost_list = self.Get('cl_update_binhost_unstable_list')
+        return Binhosts(
+            self.GetInteger('cl_update_binhost_timeout'),
+            self.Get('cl_update_binhost_revision_path'),
+            self.Get('cl_update_binhost_timestamp_path'),
+            last_ts, binhost_list,
+            self.Get('os_arch_machine'),
+            gpg=self.Get('cl_update_gpg'))
 
 class VariableClUpdateBinhostData(ReadonlyTableVariable):
     """
@@ -288,97 +309,16 @@ class VariableClUpdateBinhostData(ReadonlyTableVariable):
               "cl_update_binhost_timestamp",
               "cl_update_binhost_time"]
 
-    def get_logger(self, stub=False):
-        class StubLogger(object):
-            def info(self, message):
-                pass
-
-        if not stub and (not self.Get('cl_ebuild_phase')):
-            # проверка на chroot_path
-            return log("binhost-scan.log",
-                       filename=pathJoin(
-                           self.Get('cl_chroot_path'),
-                           "/var/log/calculate/binhost-scan.log"),
-                       formatter="%(message)s")
-        else:
-            return StubLogger()
-
     def get(self, hr=HumanReadable.No):
-        last_ts = self.Get('cl_update_last_timestamp')
-        if self.GetBool('cl_update_binhost_stable_opt_set'):
-            binhost_list = self.Get('cl_update_binhost_list')
-        else:
-            binhost_list = self.Get('cl_update_binhost_unstable_list')
-        if (self.GetBool('cl_update_binhost_stable_opt_set') and
-                not self.GetBool('cl_update_binhost_stable_set')):
-            stabilization = True
-        else:
-            stabilization = False
-        binhosts_data = Binhosts(
-            self.GetInteger('cl_update_binhost_timeout'),
-            self.Get('cl_update_binhost_revision_path'),
-            self.Get('cl_update_binhost_timestamp_path'),
-            last_ts,
-            binhost_list
-        )
-        binhost = self.Get('cl_update_binhost')
-        recheck = self.GetBool('cl_update_binhost_recheck_set')
-
-        if not recheck and binhost and binhost in binhost_list:
-            ts, t, good, downgrade = binhosts_data.get_timestamp(binhost)
-            # условие актуальности текущего сервера
-            if ts and good and not downgrade:
-                status, data = binhosts_data.check_binhost(binhost)
-                if status == binhosts_data.BinhostStatus.Success:
-                    return [[binhost, data, ts, str(t)]]
-
-        logger = self.get_logger(stub=binhosts_data.is_cache())
-        ret_val = None
-        logger.info("Started scan on: {date}, current timestamp: {ts}".format(
-            date=time.ctime(), ts=last_ts))
-        for host, ts, t, good, downgrade in binhosts_data.get_sorted():
-            if ret_val is None:
-                status, data = binhosts_data.check_binhost(host)
-            else:
-                status, data = binhosts_data.BinhostStatus.UnknownError, ""
-
-            if not good:
-                if ts == "0":
-                    reason = "FAILED (Wrong binhost)"
-                    data = ""
-                    t = 0
-                else:
-                    reason = "OUTDATED"
-            elif downgrade:
-                reason = "SKIP"
-            elif not data and ret_val is None:
-                errors = {
-                    binhosts_data.BinhostStatus.Updating: "UPDATING",
-                    binhosts_data.BinhostStatus.BadEnv:
-                        "FAILED (Bad env)",
-                    binhosts_data.BinhostStatus.EnvNotFound:
-                        "FAILED (Env not found)",
-                }
-                reason = errors.get(status, "FAILED (Unknown)")
-            else:
-                reason = ""
-
-            if ret_val is None and data and (
-                        not downgrade or stabilization):
-                ret_val = [[host, data, ts, str(t)]]
-                host = "-> %s" % host
-            logger.info("{host:<60} {speed:<7} {timestamp:<10} {reason}".format(
-                host=host, speed=float(t) / 1000.0, timestamp=ts,
-                reason=reason))
-        return ret_val or [[]]
+        return Variable.EmptyTable
 
 
 class VariableClUpdateBinhostRecheckSet(Variable):
     """
     Принудительно обновить binhost
     """
-    type = "bool"
-    value = "off"
+    type = Variable.Types.Boolean
+    value = Variable.Off
 
     opt = ["--scan"]
 
@@ -387,7 +327,7 @@ class VariableClUpdateBinhostRecheckSet(Variable):
         self.label = _("Search for the most appropriate update server")
 
 
-class VariableClUpdateBinhostHost(FieldValue, ReadonlyVariable):
+class VariableClUpdateBinhostHost(FieldValue, Variable):
     """
     Список имен прочих репозиториев
     """
@@ -396,7 +336,7 @@ class VariableClUpdateBinhostHost(FieldValue, ReadonlyVariable):
     column = 0
 
 
-class VariableClUpdateBinhostRevisions(FieldValue, ReadonlyVariable):
+class VariableClUpdateBinhostRevisions(FieldValue, Variable):
     """
     Список имен прочих репозиториев
     """
@@ -405,7 +345,7 @@ class VariableClUpdateBinhostRevisions(FieldValue, ReadonlyVariable):
     column = 1
 
 
-class VariableClUpdateBinhostTimestamp(FieldValue, ReadonlyVariable):
+class VariableClUpdateBinhostTimestamp(FieldValue, Variable):
     """
     Список имен прочих репозиториев
     """
@@ -414,6 +354,15 @@ class VariableClUpdateBinhostTimestamp(FieldValue, ReadonlyVariable):
     column = 2
 
 
+class VariableClUpdateBinhostTime(FieldValue, Variable):
+    """
+    Список имен прочих репозиториев
+    """
+    type = "list"
+    source_variable = "cl_update_binhost_data"
+    column = 3
+
+
 class VariableClUpdateLastTimestamp(ReadonlyVariable):
     """
     Текущий timestamp
@@ -424,15 +373,6 @@ class VariableClUpdateLastTimestamp(ReadonlyVariable):
         return ini.getVar('system', 'last_update') or "0"
 
 
-class VariableClUpdateBinhostTime(FieldValue, ReadonlyVariable):
-    """
-    Список имен прочих репозиториев
-    """
-    type = "list"
-    source_variable = "cl_update_binhost_data"
-    column = 3
-
-
 class VariableClUpdateBranchName(Variable):
     """
     Список доступных репозиторием
@@ -586,6 +526,29 @@ class VariableClUpdateEixupdateForce(Variable):
                 ("auto", _("If needed"))]
 
 
+class VariableClUpdateGpgForce(Variable):
+    """
+    Принудительное действие с eix-update
+    """
+    type = "choice"
+    value = "force"
+    opt = ["--check-sign"]
+    syntax = "--{choice}-check-sign"
+    metavalue = "MODE"
+    # untrusted = True
+
+    def init(self):
+        self.help = ("'force' - " + _("force check Package signature") +
+                     ",\n'skip' - " + _("skip check Package signature") +
+                     ",\n'auto' - " + _("check signature if system has public keys"))
+        self.label = _("Check Package signature")
+
+    def choice(self):
+        return [("force", _("Force")),
+                ("skip", _("Skip")),
+                ("auto", _("If has passibility"))]
+
+
 class VariableClUpdateOtherSet(Variable):
     """
     Обновить остальные оверлеи
@@ -811,9 +774,6 @@ class VariableClUpdateProfileDependData(ReadonlyTableVariable):
 
     def get(self, hr=HumanReadable.No):
         dv = self.Get(self.datavars)
-        # TODO:  неиспользуемая переменная возможно
-        # испольуется для инициализации
-        # url = self.Get('cl_update_profile_url').lower()
         if dv:
             if hr == HumanReadable.Yes:
                 return reversed(zip([x.capitalize() for x in dv.Get('cl_update_rep_name')],
@@ -1076,17 +1036,6 @@ class VariableClUpdateProfileBranch(Variable):
         self.label = _("Repository branch")
         self.help = _("set the repository branch")
 
-    def check(self, value):
-        pass
-        ## TODO: проверка ветки
-        # try:
-        #    url = self.Get('cl_update_profile_url')
-        #    self.Get('cl_update_profile_storage').get_profiles(url, value)
-        # except GitError as e:
-
-
-# raise VariableError(str(e))
-
 
 class VariableClProfileRepository(ReadonlyVariable):
     """
@@ -1736,6 +1685,12 @@ class VariableClUpdatePackageCache(ReadonlyVariable):
                             host, parsed_url.path.lstrip("/"),
                             "Packages")
 
+class VariableClUpdatePackageCacheSign(ReadonlyVariable):
+    """
+    Файл с подписью Packages находящийся в кэше
+    """
+    value_format = "{update.cl_update_package_cache}.asc"
+
 
 class VariableClUpdatePackageCacheSet(Variable):
     """
@@ -1745,7 +1700,10 @@ class VariableClUpdatePackageCacheSet(Variable):
 
     def get(self):
         packages_fn = self.Get('cl_update_package_cache')
-        if not path.exists(packages_fn):
+        gpg = self.Get('cl_update_gpg')
+        packages_asc_fn = self.Get('cl_update_package_cache_sign')
+        if (not path.exists(packages_fn) or
+            gpg and not path.exists(packages_asc_fn)):
             return "on"
         pi = PackagesIndex(readFile(packages_fn))
         try:
@@ -1762,7 +1720,7 @@ class VariableClUpdateWithBdepsSet(Variable):
     Добавление --with-bdeps при вызове emerge
     """
     type = Variable.Types.Boolean
-    value = Variable.Off
+    value = Variable.On
 
 
 class VariableClUpdateWithBdepsOptSet(Variable):
@@ -1800,3 +1758,49 @@ class VariableClUpdateSshkeyPath(Variable):
     Путь до ssh ключа
     """
     value = "/var/lib/calculate/id_rsa"
+
+class VariableClUpdateBadSignSet(Variable):
+    """
+    Ошибка проверки подписи Packages
+    """
+    type = Variable.Types.Boolean
+    value = Variable.Off
+
+class VariableClUpdateGpgKeys(Variable):
+    """
+    Список ключей для проверки подписи Packages
+    """
+    type = Variable.Types.List
+    value = ["/usr/share/openpgp-keys/calculate-release.asc"]
+
+class VariableClUpdateGpg(ReadonlyVariable):
+    """
+    Объект проверки подписи
+    """
+    type = Variable.Types.Object
+    value = ""
+
+    def close(self):
+        val = self.Get()
+        if val:
+            val.close()
+
+class VariableClUpdateUseDowngradeSet(ReadonlyVariable):
+    """
+    Можно использовать downgrade binhost
+    """
+    type = Variable.Types.Boolean
+
+    def get(self):
+        """
+        Выполняется переход с unstable ветки на stable
+        """
+        return (self.GetBool("cl_update_binhost_stable_opt_set") and 
+                not self.GetBool("cl_update_binhost_stable_opt_set"))
+
+
+class VariableClUpdatePortageBinhostPath(Variable):
+    """
+    Файл в котором хранится параметр portage PORTAGE_BINHOST
+    """
+    value = "/etc/portage/make.conf/binhost"
